@@ -13,12 +13,14 @@ from typing import Mapping
 
 import warnings
 
+from liquid.context import Undefined
 from liquid.mode import Mode
 from liquid.tag import Tag
 from liquid.template import BoundTemplate
 from liquid.lex import get_lexer
 from liquid.stream import TokenStream
 from liquid.parse import get_parser
+from liquid.utils import LRUCache
 
 from liquid import ast
 from liquid import builtin
@@ -27,8 +29,6 @@ from liquid import loaders
 from liquid.exceptions import Error
 from liquid.exceptions import lookup_warning
 
-from liquid.utils import LRUCache
-
 
 # pylint: disable=too-many-instance-attributes
 class Environment:
@@ -50,6 +50,9 @@ class Environment:
         `Mode.LAX`, `Mode.WARN` or `Mode.STRICT`.
     :param loader: A template loader. If you want to use the builtin "render" or
         "include" tags, a loader must be configured.
+    :param undefined: A subclass of :class:`Undefined` that represents undefined values.
+    :param strict_filters: If `True`, the default, will raise an exception upon finding
+        an undefined filter. Otherwise undefined filters are silently ignored.
     :param globals: A mapping that will be added to the context of any template loaded
         from this environment.
     """
@@ -64,6 +67,8 @@ class Environment:
         strip_tags: bool = False,
         tolerance: Mode = Mode.STRICT,
         loader: Optional[loaders.BaseLoader] = None,
+        undefined: Type[Undefined] = Undefined,
+        strict_filters: bool = True,
         globals: Optional[Mapping[str, object]] = None,
     ):
         self.tag_start_string = tag_start_string
@@ -74,6 +79,13 @@ class Environment:
         self.loader = loader or loaders.DictLoader({})
         self.globals: Mapping[str, object] = globals or {}
 
+        # The undefined type. When an identifier can not be resolved, the returned value
+        # is ``Undefined`` or a subclass of ``Undefined``.
+        self.undefined = undefined
+
+        # Indicates if an undefined filter should raise an exception or be ignored.
+        self.strict_filters = strict_filters
+
         # Tag register.
         self.tags: Dict[str, Tag] = {}
 
@@ -109,17 +121,16 @@ class Environment:
     def add_filter(self, name: str, func: Callable[..., Any]) -> None:
         """Register a filter function with the environment.
 
-        Args:
-            name: The filter's name. Does not need to match the function name.
-                This is what you'll use to apply the filter to an expression in
-                a liquid template.
-            func: Any callable that accepts at least one argument, the result
-                of the expression the filter is applied to.
+        :param name: The filter's name. Does not need to match the function name. This
+            is what you'll use to apply the filter to an expression in a liquid
+            template.
+        :param func: Any callable that accepts at least one argument, the result of the
+            expression the filter is applied to.
 
-                If the filter needs access to the environment or render
-                context, you probably want to make `func` a class that inherits
-                from `liquid.Filter`, and override the `__call__` method. All
-                builtin filters are implemented in this way.
+            If the filter needs access to the environment or render context, you
+            probably want to make `func` a class that inherits from `liquid.Filter`, and
+            override the `__call__` method. All builtin filters are implemented in this
+            way.
         """
         self.filters[name] = func
 
@@ -153,14 +164,12 @@ class Environment:
         `name` and `path` are passed to the `Template` constructor and will be
         used to populate the template's `TemplateDrop`.
 
-        Args:
-            source: Liquid template source.
-            name: Name of the template.
-            path: Location of the template.
-            globals: A mapping of context variables made available every time
-                the template is rendered.
-        Returns:
-            A parsed template ready to be rendered.
+        :param source: Liquid template source.
+        :param name: Name of the template.
+        :param path: Location of the template.
+        :param globals: A mapping of context variables made available every time the
+            template is rendered.
+        :returns: A parsed template ready to be rendered.
         """
         parse_tree = self.parse(source)
         return self.template_class(
@@ -177,13 +186,11 @@ class Environment:
     ) -> BoundTemplate:
         """Load and parse a template using the configured loader.
 
-        Args:
-            name: The template's name. The loader is responsible for
-                interpretting the name.
-            globals: A mapping of context variables made available every time
-                the resulting template is rendered.
-        Returns:
-            A parsed template ready to be rendered.
+        :param name: The template's name. The loader is responsible for interpretting
+            the name.
+        :param globals: A mapping of context variables made available every time the
+            resulting template is rendered.
+        :returns: A parsed template ready to be rendered.
         """
         template = self.cache.get(name)
 
@@ -255,6 +262,8 @@ def Template(
     statement_end_string: str = r"}}",
     strip_tags: bool = False,
     tolerance: Mode = Mode.STRICT,
+    undefined: Type[Undefined] = Undefined,
+    strict_filters: bool = True,
     globals: Optional[Mapping[str, object]] = None,
 ) -> BoundTemplate:
     """A :class:`BoundTemplate` factory.
@@ -274,6 +283,9 @@ def Template(
         `Mode.LAX`, `Mode.WARN` or `Mode.STRICT`.
     :param loader: A template loader. If you want to use the builtin "render" or
         "include" tags, a loader must be configured.
+    :param undefined: A subclass of :class:`Undefined` that represents undefined values.
+    :param strict_filters: If `True`, the default, will raise an exception upon finding
+        an undefined filter. Otherwise undefined filters are silently ignored.
     :param globals: A mapping that will be added to the context of any template loaded
         from this environment.
     """
@@ -284,6 +296,8 @@ def Template(
         statement_end_string,
         strip_tags,
         tolerance,
+        undefined,
+        strict_filters,
     )
 
     return env.from_string(source, globals=globals)
