@@ -56,6 +56,10 @@ class FilterArgumentError(Error):
     """Exception raised when a filters arguments are invalid."""
 
 
+class FilterValueError(Error):
+    """Exception raised when a filters value is invalid."""
+
+
 class TemplateNotFound(Error):
     """Excpetions raised when a template could not be found."""
 
@@ -69,6 +73,10 @@ class ContextDepthError(Error):
     """Exception raised when the maximum context depth is reached."""
 
 
+class UndefinedError(Error):
+    """Exception raised by the StrictUndefined type."""
+
+
 class BreakLoop(LiquidInterrupt):
     """Exception raised when a BreakNode is rendered."""
 
