@@ -428,6 +428,11 @@ class LiquidLexerTestCase(TestCase):
                 "collection",
                 [Token(1, TOKEN_IDENTIFIER, "collection")],
             ),
+            Case(
+                "lone identifier with a hyphen",
+                "main-collection",
+                [Token(1, TOKEN_IDENTIFIER, "main-collection")],
+            ),
             Case(
                 "chained identifier",
                 "collection.products",
