@@ -227,6 +227,16 @@ class RenderTestCases(unittest.TestCase):
                     template=r"{{ false | default: 'bar', allow_false: false }}",
                     expect="bar",
                 ),
+                Case(
+                    description="unexpected `join` filter left value passes through",
+                    template=r"{{ 12 | join: '#' }}",
+                    expect="12",
+                ),
+                Case(
+                    description="join a string",
+                    template=r"{{ 'a,b' | join: '#' }}",
+                    expect="a,b",
+                ),
             ]
         )
 
@@ -324,6 +334,11 @@ class RenderTestCases(unittest.TestCase):
                     template=r"{% echo today | date: '%d/%m/%Y' %}",
                     expect=datetime.date.today().strftime(r"%d/%m/%Y"),
                 ),
+                Case(
+                    description="index of undefined",
+                    template=r"{{ nosuchthing[0] }}",
+                    expect="",
+                ),
             ]
         )
 
@@ -959,6 +974,12 @@ class RenderTestCases(unittest.TestCase):
                     "section": {"settings": {"menu": "main"}},
                 },
             ),
+            Case(
+                description="loop over undefined",
+                template=r"{% for tag in nosuchthing %}{{ tag }}{% endfor %}",
+                expect="",
+                globals={},
+            ),
         ]
 
         self._test(test_cases)
